import SwiftUI
import HealthKit

struct ActivityDetailView: View {

    @EnvironmentObject
    var database: HealthDatabase

    let activity: HKWorkoutActivity

    @State var locations: [LocationSample] = []

    @State var sampleCount: Int = 0

    var body: some View {
        List {
            DetailRow("UUID", value: activity.uuid)
            DetailRow("Activity", value: activity.workoutConfiguration.activityType)
            DetailRow("Location", value: activity.workoutConfiguration.locationType)
            DetailRow("Swimming Location", value: activity.workoutConfiguration.swimmingLocationType)
            DetailRow("Lap Length", value: activity.workoutConfiguration.lapLength)
            DetailRow("Start", date: activity.startDate)
            DetailRow("End", date: activity.endDate)
            DetailRow("Duration", duration: activity.duration)
            DetailRow("Metadata", value: activity.metadata)
            if !locations.isEmpty {
                NavigationLink(value: locations) {
                    DetailRow("Locations", value: "\(locations.count)")
                }
            } else {
                DetailRow("Locations", value: "0")
            }
            if sampleCount != 0 {
                NavigationLink {
                    ActivitySamplesView(activity: activity)
                        .environmentObject(database)
                } label: {
                    DetailRow("Samples", value: "\(sampleCount)")
                }
            } else {
                DetailRow("Samples", value: "0")
            }
        }
        .navigationTitle("Activity")
        .navigationDestination(for: [LocationSample].self) { locations in
            LocationSampleListView(samples: locations)
        }
        .onAppear(perform: load)
    }

    private func load() {
        Task {
            do {
                let samples = try database.locationSamples(for: activity)
                    .sorted { $0.timestamp }
                let sampleCount = try database.sampleCount(for: activity)
                DispatchQueue.main.async {
                    self.locations = samples
                    self.sampleCount = sampleCount
                }
            } catch {
                print("Failed to load location samples for activity: \(error)")
            }
        }
    }
}

#Preview {
    NavigationStack {
        ActivityDetailView(activity: .mock1)
        .environmentObject(HealthDatabase.mock())
    }
}
